/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

// no rotation
class Circle extends Vertex implements Sketchable
{
    public static final double TOLERANCE = 15;

    public double radius;

    public Circle(double x, double y, double r)
    {
	super(x, y);
	DEFAULT_COLOR = Color.red.darker().darker();
	changeColor(DEFAULT_COLOR);
	update(x, y, r);
	super.sketchedObjects.add(this);
    }

    public void update(double x, double y, double r)
    {
	radius = r;
	this.x = x;
	this.y = y;
    }

    public boolean inBound(double x, double y)
    {
	double dist = rad2D(this.x, this.y, x, y);
	return Math.abs(dist - radius) <= TOLERANCE;
    }

    public double rad2D(double px, double py, double qx, double qy)
    {
	return Math.sqrt(sqr(px - qx) + sqr(py - qy));
    }

    public double sqr(double r)
    {
	return r*r;
    }

    public void draw(Graphics g, DrawingArea drawArea)
    {
	Vertex pp = drawArea.transform(this.x-radius, this.y+radius);
	Vertex qq = drawArea.transform(this.x+radius, this.y-radius);

	//changeColor(Color.red.darker().darker());
	g.setColor(getColor());

	// java-ism
	g.drawOval(Math.round((float)pp.x), Math.round((float)pp.y), 
		   Math.round((float)Math.abs(qq.x-pp.x)), Math.round((float)Math.abs(qq.y-pp.y)));

    }

    public void mouseDragged(double x, double y, Vertex anchor, 
			     int modifier) 
    { 
	double newRad = rAnchor;
	double newX   = cAnchor.x;
	double newY   = cAnchor.y;

	newRad = Math.sqrt(distSqr(cAnchor.x, cAnchor.y, cAnchor.z, x, y, z));
	update(newX, newY, newRad);
    }

    Vertex cAnchor = new Vertex(0, 0);
    double rAnchor;
    public void mousePressed(double x, double y, int modifier) 
    { 
	cAnchor.x = this.x;
	cAnchor.y = this.y;
	cAnchor.z = this.z;
	rAnchor   = radius;
    }

    public Vertex addPoint(double x, double y)
    {
	return null;
    }

    public Vertex updateCoord(double x, double y)
    {
	radius = rad2D(this.x, this.y, x, y);
	return null;
    }

    public boolean remove()
    {
	super.sketchedObjects.remove(this);
	return true;
    }

    public String toString()
    {
	return "Circ r="+radius+" "+super.toString();
    }

}

