/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;
import java.awt.*;
import java.util.*;
import javax.swing.*;

public class Vertex implements VertexListener
{
    public static ArrayList beacons          = new ArrayList();
    public static ArrayList sketchedObjects  = new ArrayList();
    public static ArrayList streamer         = new ArrayList();
    public static ArrayList trackedObjects   = new ArrayList();

    public static final double TOLERANCE = 5;

    public double x;
    public double y;
    public double z;

    protected Vertex transformed_xyz;
    protected boolean modified = false;

    VertexListener vertexListener = null;
    
    public Vertex(double x, double y) 
    {
	this.x = x; this.y = y;
	DEFAULT_COLOR = Color.blue;
	changeColor(DEFAULT_COLOR);
    }

    public static void clearList()
    {
	beacons.clear();
	sketchedObjects.clear();
	streamer.clear();
	trackedObjects.clear();
    }

    public boolean inBound(double x, double y)
    {
	//System.out.println(this.x+" "+this.y);
	return
	    Math.abs(this.x - x) < TOLERANCE &&
	    Math.abs(this.y - y) < TOLERANCE;
    }

    public static void drawAll(Graphics g, DrawingArea drawArea)
    {
	for(int i = 0; i < sketchedObjects.size(); i++) {
	    ((Vertex)sketchedObjects.get(i)).draw(g, drawArea);
	}	
	for(int i = 0; i < beacons.size(); i++) {
	    ((Vertex)beacons.get(i)).draw(g, drawArea);
	}
	for(int i = 0; i < streamer.size(); i++) {
	    ((Vertex)streamer.get(i)).draw(g, drawArea);
	}
	for(int i = 0; i < trackedObjects.size(); i++) {
	    ((Vertex)trackedObjects.get(i)).draw(g, drawArea);
	}
    }

    public  Color DEFAULT_COLOR;
    private Color color;

    public Color getColor() { return color; }

    public void changeColor(VertexListener vl, Color c) {
	color = c;
	if(vl != null)
	    vl.changeColor(c);	
    }

    public void changeColor(Color c) {
	changeColor(vertexListener, c);
    }

    public void draw(Graphics g, DrawingArea drawArea) 
    {
	modified = false;
	g.setColor(color);

	Vertex v = drawArea.transform(x, y);

	//System.out.println(v.x+" : "+v.y);

	g.drawString("("+Util.textOf(x, 2)+", "+Util.textOf(y, 2)+")",
		     Math.round((float)(v.x))+2,
		     Math.round((float)(v.y))+ 12);


	g.fillOval(Math.round((float)(v.x - DrawingArea.nodeSizeDiv2)), 
		   Math.round((float)(v.y - DrawingArea.nodeSizeDiv2)), 
		   DrawingArea.nodeSize, 
		   DrawingArea.nodeSize);

	transformed_xyz = v;
    }

    public String toString()
    {
	return toString(vertexListener);
    }

    public String toString(VertexListener vl) 
    {
	if(vl != null)
	    return vl.toString();
	else
	    return "("+this.x+","+this.y+")";
    }

    public double distSqr(Vertex v)
    {
	return distSqr(x, y, z, v.x, v.y, v.z);
    }

    public double distSqr(double xx, double yy, double zz)
    {
	return distSqr(x, y, z, xx, yy, zz);
    }

    public double distSqr(double X, double Y, double Z, double XX, double YY, double ZZ)
    {
	//System.out.println(X+" "+Y+" "+Z+" "+XX+" "+YY+" "+ZZ);
	return (X - XX)*(X - XX) + (Y - YY)*(Y - YY) + (Z-ZZ)*(Z-ZZ);
    }

    public boolean equals(Object o)
    {
	return this == o;
    }

    public static Vertex findObject(double x, double y)
    {
	Iterator it = sketchedObjects.iterator();
	while(it.hasNext()) {
	    Vertex obj = (Vertex) it.next();
	    if(obj.inBound(x, y)) {
		return obj;
	    }
	}

	it = beacons.iterator();
	while(it.hasNext()) {
	    Vertex obj = (Vertex) it.next();
	    if(obj.inBound(x, y))
		return obj;
	}
	return null;
    }

    public static void removeAll()
    {
	// we cannot use Iterator here because there are "concurrent"
	// modification to the iterated data structure
	int i = 0;
	while (i < sketchedObjects.size()) {
	    int s = sketchedObjects.size();
	    
	    Vertex obj = (Vertex) sketchedObjects.get(i);
	    if(!obj.remove())
		i++;
	    else {
		Util.assert_nonjava(sketchedObjects.size() < s, 
				    "Vertex::removeAll() fatal error");
		// if remove() returned true, there MUST be something removed
		// from sketchedObjects array, or else infinite loop will be imminent
	    }
	}

	// we don't bother to check 
	/*
	  it = beacons.iterator();
	  while(it.hasNext()) {
	  Vertex obj = (Vertex) it.next();
	  obj.remove();
	  }
	*/
    }

    public void mousePressed(double x, double y, int modifier)
    {
	return;
    }

    public void mouseDragged(double x, double y, Vertex anchor, 
			     int modifier)
    {
	this.x = x; this.y = y;
	modified = true;
    }

    public void mouseReleased(double x, double y)
    {
	return;
    }

    // by default, the Vertex object is not removable.  
    // sub class must override this method to make it removeable
    // thus far, Beacons and SquareVertex are not removable
    public boolean remove()
    {
	if(vertexListener != null) {
	    return vertexListener.remove();
	}
	return false;
    }

    public void setVertexListener(VertexListener vrl)
    {
	vertexListener = vrl;
    }

    public Object clone()
    {
	return new Vertex(x, y);
    }
}
