/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader.parse;

import cricketdaemon.Util;
import java.util.*;

/**
 * Maps the default field type value to the corresponding FieldParser.
 *
 * @author Allen Miu
 */
public class ParseMap
{
    /* Type values */
    //    public final static int NULL       = 0; // should never use this
    public final static int SPACE      = 1;
    public final static int BEACONID   = 2;  // deprecated
    public final static int DISTANCE   = 3;
    public final static int CARTESIAN_COORD = 4;
    public final static int PHASEARRAY = 5;
    public final static int VERSION    = 6;
    public final static int DURATION   = 7;
    public final static int ORIENT     = 8; // hack to propagate orientation info from SimReader
    public final static int LISTENTIME = 9; // timestamp from listener
    public final static int UNIQUEID   = 10;
    public final static int LOCALTIME  = 11; // timestamp from gpsd
 
    /**
     * Field types that do not change value across multiple readings from
     * the same beacon.
     */
    public final static int[] persistentFields = 
    { SPACE, BEACONID, CARTESIAN_COORD };


   /** 
     * The ith entry corresponds to the type that has a value i, 
     * as defined above. 
     */
    public static FieldParser[] map =
    { 
	null,                       // NULL
	FieldParser_ASCII.instance,    // SPACE
	FieldParser_Integer.instance,  // BEACONID     
	FieldParser_Integer.instance,  // DISTANCE
	FieldParser_CartesianCoord.instance, // COORD
	FieldParser_PhaseArray.instance, 
	FieldParser_Float.instance, // VERSION
	FieldParser_Integer.instance, // PULSE DURATION
	FieldParser_Float.instance, // ORIENT	
	FieldParser_Long.instance,  
	FieldParser_ASCII.instance,  
	FieldParser_Long.instance,  
    };

    public static final int NUM_TYPES = map.length;

    public static Hashtable GPSDMap; // must be consistent with "map"
    static {
	GPSDMap = new Hashtable();
	GPSDMap.put("space", new Integer(SPACE)); 
	GPSDMap.put("id",    new Integer(BEACONID));
	GPSDMap.put("dist",  new Integer(DISTANCE));
	GPSDMap.put("coord", new Integer(CARTESIAN_COORD));
	GPSDMap.put("phase", new Integer(PHASEARRAY));
	GPSDMap.put("ver", new Integer(VERSION));
	GPSDMap.put("duration", new Integer(DURATION));
	GPSDMap.put("orient", new Integer(ORIENT));
	GPSDMap.put("time", new Integer(LISTENTIME));
	GPSDMap.put("uniqueid", new Integer(UNIQUEID));

	// to support V2 attrib=value format
	GPSDMap.put("SP", new Integer(SPACE)); 
	GPSDMap.put("DB",  new Integer(DISTANCE));
	GPSDMap.put("PC", new Integer(CARTESIAN_COORD));
	GPSDMap.put("DR", new Integer(DURATION));
	GPSDMap.put("VR", new Integer(VERSION));
	GPSDMap.put("TS", new Integer(LISTENTIME));
	GPSDMap.put("ID", new Integer(UNIQUEID));
	GPSDMap.put("LT", new Integer(LOCALTIME));
    }

    public static int typeToFieldID(String type)
    {
	Object o = GPSDMap.get(type);
	if(o == null)
	    return -1;
	else
	    return ((Integer) o).intValue();
    }
}

